
// MethodVisual.java
// Andrew Davison, March 2009, ad@fivedots.coe.psu.ac.th

/* Generates and stores the Whorld visualization parameters
   which are used to draw the visual for the method
   className.methodName(). Sends these parameters to Whorld
   when show() is called

   The parameter generation uses the byte size of the method to
   decide how many of the parameters will have (simple) default
   values.

   Parameter details are stored in the WParams enumerated type.
*/


import javax.sound.midi.*;
import java.util.*;


public class MethodVisual
{
  // byte sizes at which code 'complexity' changes
  private static final int MAX_SIMPLE = 40;   // 0-39 is simple code
  private static final int MAX_MEDIUM = 80;   // 40-79 is medium; 80+ is complex

  // channel 1 chosen to receive messages
  private static final int CHANNEL = 0;


  private String className, methodName;

  private int ringSides, starFactor, pinWheel, oddCurve, evenCurve,  
              oddShear, evenShear, rotateSpeed, skewAngle, skewRadius;
     // Whorld parameter values used to specify the visualization



  public MethodVisual(String classNm, String methodNm, int methodSize)
  { 
    className = classNm;
    methodName = methodNm;
    System.out.println("  Adding " + className + "." + methodName + "()");

    // always generate these parameters
    ringSides = WParams.RING_SIDES.genValue();

    starFactor = WParams.STAR_FACTOR.genValue();
    pinWheel = WParams.PIN_WHEEL.genValue();

    oddCurve = WParams.ODD_CURVE.genValue();

    rotateSpeed = WParams.ROTATE_SPEED.genValue();


    /* use defaults for these parameters, if the code is simple
       which is judged by looking at the byte size of the method */

    // medium complexity
    evenCurve = (methodSize >= MAX_SIMPLE)? WParams.EVEN_CURVE.genValue() :
                                            WParams.EVEN_CURVE.getDefault();

    oddShear = (methodSize >= MAX_SIMPLE)? WParams.ODD_SHEAR.genValue() :
                                           WParams.ODD_SHEAR.getDefault();

    evenShear = (methodSize >= MAX_SIMPLE)? WParams.EVEN_SHEAR.genValue() :
                                            WParams.EVEN_SHEAR.getDefault();
                 
    // above medium (high) complexity
    skewAngle = (methodSize >= MAX_MEDIUM)? WParams.SKEW_ANGLE.genValue() :
                                            WParams.SKEW_ANGLE.getDefault();

    skewRadius = (methodSize >= MAX_MEDIUM)? WParams.SKEW_RADIUS.genValue() :
                                             WParams.SKEW_RADIUS.getDefault();
  } // end of MethodVisual()




  public boolean isNamed(String classNm, String methodNm)
  // is the name of this visualization classNm.methodNm()?
  {
    if (className.equals(classNm) && methodName.equals(methodNm))
      return true;
    return false;
  }  // end of isNamed()



  public void show(Receiver receiver)
  // send this method's visualization parameters over to Whorld
  {
    System.out.println("  Showing " + className + "." + methodName + "()");

    sendParameter(receiver, WParams.RING_SIDES, ringSides);

    sendParameter(receiver, WParams.STAR_FACTOR, starFactor);
    sendParameter(receiver, WParams.PIN_WHEEL, pinWheel);

    sendParameter(receiver, WParams.ODD_CURVE, oddCurve);
    sendParameter(receiver, WParams.EVEN_CURVE, evenCurve);

    sendParameter(receiver, WParams.ODD_SHEAR, oddShear);
    sendParameter(receiver, WParams.EVEN_SHEAR, evenShear);
                 
    sendParameter(receiver, WParams.ROTATE_SPEED, rotateSpeed);

    sendParameter(receiver, WParams.SKEW_ANGLE, skewAngle);
    sendParameter(receiver, WParams.SKEW_RADIUS, skewRadius);
  }  // end of show()



  private void sendParameter(Receiver receiver, WParams param, int value)
  /* send parameter value for param over to Whorld as a MIDI CONTROL_CHANGE
     message. */
  {
    int controller = param.getControllerID();  // get controller used for this parameter

    // System.out.println("Sending " + value + " to controller " + controller);
    ShortMessage message = new ShortMessage();
    try {
      message.setMessage(ShortMessage.CONTROL_CHANGE, CHANNEL, controller, value);
      receiver.send(message, -1);
    } 
    catch (InvalidMidiDataException e) {
      System.out.println(e);
    }
  } // end of sendParameter()



} // end of MethodVisual class
