
// SendParam.java
// Andrew Davison, March 2009, ad@fivedots.coe.psu.ac.th

/* Send a parameter value to the Whorld visualizer, which is accessed
   via the LoopBe1 MIDI virtual input port, and controlled using
   MIDI CONTROL_CHANGE messages.

   Software available at:
   * LoopBe1 virtual MIDI driver (http://www.nerds.de/en/loopbe1.html)
   * Whorld (http://www.churchofeuthanasia.org/whorld/)

   Usage:
      java SendParam <parameter name> <byte value>
   or
      java SendParam
          - lists the known parameter names

   The parameter names are stored in the paramNames[] array, and mapped
   to controller IDs in main().

   The value must be a byte (0-127), but it's effect on the parameter depends
   on the value range specified in Whorld's MIDI setup window. The resulting value
   is best checked by looking at Whorld's Numbers window,
   and by observing how the Whorld pattern changes.

   The parameters are described in the Whorld documentation 
   (http://www.churchofeuthanasia.org/whorld/Help/printable/WhorldHelp.htm), 
    but briefly:
      * ringSides:    the number of sides in a ring

      * rotateSpeed:  rotation per frane, in degrees

      * starFactor:   makes edges 'fold out' into a triangle, or 'fold in'
      * pinWheel:     shift star factor points to the left or right

      * oddCurve:     curvature of star factor points (called odd vertices)
      * evenCurve:    curvature of star factor insets (called even vertices)

      * oddShear:     makes the curvature at odd vertices asymmetrical
      * evenShear:    makes the curvature at even vertices asymmetrical

      * skewRadius:  offset of each ring's origin per frame

      * skewAngle:  the direction of the skew shift
                     0 degress == south; 90 degrees == east

*/


import javax.sound.midi.*;
import java.util.*;


public class SendParam
{

  // channel 1 chosen to receive messages
  private static final int CHANNEL = 0;

  // parameter names used in Whorld
  private static final String[] paramNames =
       { "ringSides", "rotateSpeed", "starFactor", "pinWheel", 
		 "oddCurve", "evenCurve", "oddShear", "evenShear",      
		"skewRadius", "skewAngle" };

  // controller IDs used to modify the Whorld parameters
  private static final int[] controllerIDs = 
       { 4, 5, 9, 10, 18, 17, 20, 19, 7, 8 };   



  public static void main(String[] args)
  { 
    if (args.length != 2) {
      System.out.println("Usage: java SendParam <parameter name> <byte value>");
      System.out.println("Possible parameter names:");
      for(String nm : paramNames)
        System.out.print(" " + nm);
      System.out.println();
      System.exit(0);
    }

    int controllerID = getID(args[0]);
    int value = getValue(args[1]);

    Receiver receiver = findReceiver("LoopBe");
    sendMessage(receiver, controllerID, value);
    receiver.close();  
  } // end of main()



  private static int getID(String paramName)
  // convert parameter name into a controller ID
  {
    int controllerID = -1;
    for(int i=0; i < paramNames.length; i++)
      if (paramName.equals(paramNames[i])) {
        controllerID = controllerIDs[i];
        break;
      }

    if (controllerID == -1) {
      System.out.println("Unknown Whorld parameter name: " + paramName);
      System.exit(0);
    }

    return controllerID;
  }  // end of getID()



  private static int getValue(String valStr)
  // extract byte value from input
  {
    int value = 64;
    try { 
      value = Integer.parseInt(valStr); 
    }
    catch(NumberFormatException e)
    { System.out.println("Value not an integer: " + valStr); 
      System.exit(0);
    }

    if ((value < 0) || (value > 127)) {
      System.out.println("Parameter out of range (0-127); using 64");
      value = 64;
    }

    return value;
  }  // end of getValue()





  private static Receiver findReceiver(String name)
  // find (and open) a MIDI receiver whose name starts with name
  {
    System.out.println("Searching for device: " + name);
    try {
      MidiDevice.Info[] devices = MidiSystem.getMidiDeviceInfo();
      for(MidiDevice.Info devInfo : devices)
        if (devInfo.getName().startsWith(name)) {
          MidiDevice dev = MidiSystem.getMidiDevice(devInfo);
          if (dev.getMaxReceivers() != 0) {   // must be a receiver
            System.out.println("Found: " + devInfo.getDescription());
            dev.open();   
            System.out.println("Opening receiver");
            return dev.getReceiver();
          }
        }
      System.out.println("Device not found");
      System.exit(0);
    }
    catch (MidiUnavailableException e){
      System.out.println(e);
      System.exit(0);
    }
    return null;
  }  // end of findReceiver()




  private static void sendMessage(Receiver receiver, int controllerID, int value) 
  // send a CONTROL_CHANGE message to the controller, setting it to be value 
  {
    System.out.println("Sending " + value + " to controller " + controllerID);

    ShortMessage message = new ShortMessage();
    try {
      message.setMessage(ShortMessage.CONTROL_CHANGE, CHANNEL, controllerID, value);
      receiver.send(message, -1);
    } 
    catch (InvalidMidiDataException e) {
      System.out.println(e);
    }
  } // end of sendMessage()



} // end of SendParam class
