
// Visualizer.java
// Andrew Davison, March 2009, ad@fivedots.coe.psu.ac.th

/* The interface for using the Whorld visualizer, which is accessed
   via the LoopBe 1 MIDI virtual input port, and controlled using
   MIDI CONTROL_CHANGE messages.

   The software is available at:
   * LoopBe1 virtual MIDI driver (http://www.nerds.de/en/loopbe1.html)
   * Whorld (http://www.churchofeuthanasia.org/whorld/)

   This class also stores a list of MethodVisual object, one for
   each method visualization used in Whorld.

*/


import javax.sound.midi.*;
import java.util.*;


public class Visualizer
{
  // (start of) name of MIDI device linked to Whorld
  private static final String MIDI_NAME = "LoopBe";
  
  // channel 1 chosen to receive messages
  private static final int CHANNEL = 0;

  // Whorld controllers IDs
  private static final int PAUSE = 51;
  private static final int CLEAR = 52;
  

  private Receiver receiver;
  private ArrayList<MethodVisual> visuals;

  private boolean isPaused = false;


  public Visualizer()
  { 
    receiver = findReceiver(MIDI_NAME);
    pause(true);
    clear();
    visuals = new ArrayList<MethodVisual>();
  } // end of Visualizer()




  private Receiver findReceiver(String name)
  // find (and open) a MIDI receiver whose name starts with name
  {
    System.out.println("Searching for device: " + name);
    try {
      MidiDevice.Info[] devices = MidiSystem.getMidiDeviceInfo();
      for(MidiDevice.Info devInfo : devices)
        if (devInfo.getName().startsWith(name)) {
          MidiDevice dev = MidiSystem.getMidiDevice(devInfo);
          if (dev.getMaxReceivers() != 0) {   // must be a receiver
            System.out.println("Found: " + devInfo.getDescription());
            dev.open();   
            System.out.println("Opening receiver");
            return dev.getReceiver();
          }
        }
      System.out.println("Device not found");
      System.exit(0);
    }
    catch (MidiUnavailableException e){
      System.out.println(e);
      System.exit(0);
    }
    return null;
  }  // end of findReceiver()



  public boolean contains(String className, String methodName)
  // do we already have a visualization for className.methodName?
  {
    for(MethodVisual vis: visuals)
      if (vis.isNamed(className, methodName))
        return true;
    return false;
  }  // end of contains()



  public boolean add(String className, String methodName, int methodSize)
  /* Add a visualization for classname.methodName(), using the byte size
     of the method to create the visual, but only if no such visualization
     already exists */
  {
    for(MethodVisual vis: visuals)
      if (vis.isNamed(className, methodName)) {
        System.out.println("Method " + className + "." + methodName + 
                                                      "() already exists");
        return false;
      }
    visuals.add( new MethodVisual(className, methodName, methodSize) );
    return true;
  }  // end of add()



  public void show(String className, String methodName)
  // show the visualization for className.methodName()
  {
    for(MethodVisual vis: visuals)
      if (vis.isNamed(className, methodName)) {
        if (isPaused)
          pause(false);
        vis.show(receiver);
        return;
     }
  }  // end of show()



  private void pause(boolean b)
  // pause/resume the Whorld Visualizer
  {
    isPaused = b;
    if (isPaused)
      System.out.println("Visualizer Paused");
    else
      System.out.println("Visualizer Resumed");

    int val = (isPaused) ? 127 : 0;     // 127 == on; 0 == off
    sendMessage(receiver, PAUSE, val);
  }  // end of pause()


  private void clear()
  // clear the Whorld visualizer window
  // must switch clear off before on will work
  { 
    System.out.println("Clearing the Visualizer");
    sendMessage(receiver, CLEAR, 0);    // 0 == off
    sendMessage(receiver, CLEAR, 127);    // 127 == on
  }



  private void sendMessage(Receiver receiver, int controller, int value) 
  // send a CONTROL_CHANGE message to the controller, setting it to be value 
  {
    // System.out.println("Sending " + value + " to controller " + controller);
    ShortMessage message = new ShortMessage();
    try {
      message.setMessage(ShortMessage.CONTROL_CHANGE, CHANNEL, controller, value);
      receiver.send(message, -1);
    } 
    catch (InvalidMidiDataException e) {
      System.out.println(e);
    }
  } // end of sendMessage()



  public void closeDown()
  {  
    System.out.println("Closing link to Visualizer");
    // pause(true);
    receiver.close();  
  }


} // end of Visualizer class
