
// WParams.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, March 2009

/* Whorld parameter names, together with their
      controller ID used by Whorld, min and max values

   A parameter is set in Whorld by sending a MIDI control message
   to the specified controller ID, with a value stored in a 1 byte.

   The minimum and maximum values are a subset of the full byte
   range 0-127 for each controller value. The range in comments
   are how the byte range is interpreted by Whorld, which is based on 
   the MIDI default settings. 

   I tested the range by checking what Whorld
   reported in its Numbers dialog box when I set a particular controller.
   (see SendParam.java)

   The parameters are described in the Whorld documentation 
   (http://www.churchofeuthanasia.org/whorld/Help/printable/WhorldHelp.htm), 
    but briefly:
      * RING_SIDES:    the number of sides in a ring

      * ROTATE_SPEED:  rotation per frane, in degrees

      * STAR_FACTOR:   makes edges 'fold out' into a triangle, or 'fold in'
      * PIN_WHEEL:     shift star factor points to the left or right

      * ODD_CURVE:     curvature of star factor points (called odd vertices)
      * EVEN_CURVE:    curvature of star factor insets (called even vertices)

      * ODD_SHEAR:     makes the curvature at odd vertices asymmetrical
      * EVEN_SHEAR:    makes the curvature at even vertices asymmetrical

      * SKEW_RADIUS:  offset of each ring's origin per frame

      * SKEW_ANGLE:  the direction of the skew shift
                     0 degress == south; 90 degrees == east
*/


import java.util.*;

public enum WParams 
{
  RING_SIDES    (4,  62, 66),    // 3 to 7 sides
  ROTATE_SPEED  (5,  51, 77),    // -1 to 1 ; clockwise --> anti-clockwise
  STAR_FACTOR   (9,  43, 85),    // -1 to 1  ; fold-in --> fold-out
  PIN_WHEEL     (10, 43, 85),    // -1 to 1  ; right --> left
  ODD_CURVE     (18, 71, 85),    // 0.3333 to 1  // positive curvature
  EVEN_CURVE    (17, 71, 85),    // 0.3333 to 1
  ODD_SHEAR     (20, 85, 106),   // 1 to 2    // twists the curvature
  EVEN_SHEAR    (19, 85, 106),   // 1 to 2
  SKEW_RADIUS   (7,  53, 75),    // -0.35 to 0.35 ; up to down
  SKEW_ANGLE    (8,  0,  127);   // -180 to 180; any direction

  private int controllerID, min, max;


  WParams(int controllerID, int min, int max) 
  {
    this.controllerID = controllerID;
    this.min = min;
    this.max = max;
  }

  public int getControllerID()
  {  return controllerID;  }

  public int getMin()
  {  return min;  }

  public int getMax()
  {  return max;  }


  public int genValue()
  // return a random value between min and max
  {
    int value = (int)(Math.random()*(max-min+1) + min);     // add 1 so max value is possible
    // System.out.println( controllerID + ". Value (" +  min + "-" + max + "): " + value);
    return value;
  }  // end of genValue()


  public int getDefault()
  // 64 means 0 for almost all the Whorld parameters (not RING_SIDES)
  {  return 64;  }

}  // end of WParams enums
