
// WhorldEvents.java

/* Robert Field and Minoru Terada, September 2005
   Iman_S, June 2008  */
// Andrew Davison, ad@fivedots.coe.psu.ac.th, March 2009

/* Monitor incoming JDI events for a program running in the JVM 
   and visualize method calls using Whorld.

   This is a simplified version of JDIEventMonitor.java from 
   my SimpleTrace.java example

   Changes to JDIEventMonitor:
     - only request events for Method entry and exit
           - no class or thread event requests sent
           - no step event or modified field events requested at run time
     - use of Visualizer class
     - no use of ShowCodes and ShowLines

     - methodEntryEvent() adds and show method info
     - methodExitEvent() shows info on appl method beting returned to 

*/


import com.sun.jdi.*;
import com.sun.jdi.request.*;
import com.sun.jdi.event.*;

import java.util.*;



public class WhorldEvents extends Thread
{
  // exclude events generated for these classes
  private final String[] excludes = { "java.*", "javax.*", "sun.*", "com.sun.*"};

  private final VirtualMachine vm;   // the JVM
  private boolean connected = true;  // connected to VM?
  private boolean vmDied;            // has VM death occurred?

  private Visualizer visualizer;


  public WhorldEvents(VirtualMachine jvm)
  {
    super("WhorldEvents");
    vm = jvm;

    visualizer = new Visualizer();
    setEventRequests();
  }  // end of WhorldEvents()



  private void setEventRequests()
  /* Create and enable the event requests for the events
     we want to monitor in the running program. */
  {
    EventRequestManager mgr = vm.eventRequestManager();

    MethodEntryRequest menr = mgr.createMethodEntryRequest(); // report method entries
    for (int i = 0; i < excludes.length; ++i)
      menr.addClassExclusionFilter(excludes[i]);
    menr.setSuspendPolicy(EventRequest.SUSPEND_EVENT_THREAD);
    menr.enable();
	
    MethodExitRequest mexr = mgr.createMethodExitRequest();   // report method exits
    for (int i = 0; i < excludes.length; ++i)
      mexr.addClassExclusionFilter(excludes[i]);
    mexr.setSuspendPolicy(EventRequest.SUSPEND_EVENT_THREAD);
    mexr.enable();
  }  // end of setEventRequests()



  public void run()
  // process JDI events as they arrive on the event queue
  { 
    EventQueue queue = vm.eventQueue();
    while (connected) {
      try {
        EventSet eventSet = queue.remove();
        for(Event event : eventSet)
          handleEvent(event);
        eventSet.resume();
      }
      catch (InterruptedException e) { }  // Ignore
      catch (VMDisconnectedException discExc) {
        handleDisconnectedException();
        break;
      }
    }
  }  // end of run()



  private void handleEvent(Event event)
  // process a JDI event
  {
    // method events
    if (event instanceof MethodEntryEvent)
      methodEntryEvent((MethodEntryEvent) event);
    else if (event instanceof MethodExitEvent)
      methodExitEvent((MethodExitEvent) event);

    // VM events
    else if (event instanceof VMStartEvent)
      vmStartEvent((VMStartEvent) event);
    else if (event instanceof VMDeathEvent)
      vmDeathEvent((VMDeathEvent) event);
    else if (event instanceof VMDisconnectEvent)
      vmDisconnectEvent((VMDisconnectEvent) event);

    else
      throw new Error("Unexpected event type");
  }  // end of handleEvent()


  private synchronized void handleDisconnectedException()
  /* A VMDisconnectedException has occurred while dealing with
     another event. Flush the event queue, dealing only
     with exit events (VMDeath, VMDisconnect) so that things 
     terminate correctly. */
  {
    EventQueue queue = vm.eventQueue();
    while (connected) {
      try {
        EventSet eventSet = queue.remove();
        for(Event event : eventSet) {
          if (event instanceof VMDeathEvent)
            vmDeathEvent((VMDeathEvent) event);
          else if (event instanceof VMDisconnectEvent)
            vmDisconnectEvent((VMDisconnectEvent) event);
        }
        eventSet.resume(); // resume the VM
      }
      catch (InterruptedException e) { }  // ignore
    }
  }  // end of handleDisconnectedException()



  // -------------------- method event handling  ---------------



  private void methodEntryEvent(MethodEntryEvent event)
  // entered a method but no code executed yet
  { 
    Method meth = event.method();
    String className = meth.declaringType().name();
    String methodName = meth.name();
    int methodSize = meth.bytecodes().length;

    System.out.println("===>> " + className +  "." + methodName +"()" +
                                " method size (bytes) = " + methodSize);

    visualizer.add(className, methodName, methodSize);
    visualizer.show(className, methodName);
  }  // end of methodEntryEvent()



  private void methodExitEvent(MethodExitEvent event)
  // all code in the method has been executed, and we are about to return
  /* Print info on application method that will be returned to;
     this may not be the next frame in the call stack */
  {
    ThreadReference thr = event.thread();
    try {
      int numFrames = thr.frameCount();

      // System.out.println("No. frames: " + numFrames);
      if (numFrames == 1)    // there's no calling frame
        return;

      StackFrame returnFrame = null;
      String className, methodName;

      // search down through the call stack, starting at frame 1
      for (int i=1; i < numFrames; i++) {
        returnFrame = thr.frame(i);   // examine a frame
        if (returnFrame != null) {
          Location loc = returnFrame.location();
          Method meth = loc.method();
          className = meth.declaringType().name();
          methodName = meth.name();

          // if the function has a visualization, show it
          if (visualizer.contains(className, methodName)) {
            System.out.println("<<=== " + className + "." + methodName +"()");
            visualizer.show(className, methodName);
            return;
          }
        }
      }
    }
    catch (IncompatibleThreadStateException e) 
    {  System.out.println(e); }
  }  // end of methodExitEvent()


  // ---------------------- VM event handling ----------------------------------

  private void vmStartEvent(VMStartEvent event)
  /* Notification of initialization of a target VM. This event is received 
     before the main thread is started and before any application code has 
     been executed. */
  { vmDied = false;
    System.out.println("-- VM Started --"); 
  }


  private void vmDeathEvent(VMDeathEvent event)
  // Notification of VM termination
  { vmDied = true;
    System.out.println("-- The application has exited --");
  }


  private void vmDisconnectEvent(VMDisconnectEvent event)
  /* Notification of disconnection from the VM, either through normal termination 
     or because of an exception/error. */
  { connected = false;
    if (!vmDied)
      System.out.println("-- The application has been disconnected --");
  }

}  // end of WhorldEvents class
